#!/usr/bin/env python
# coding=utf8
#
#
# This file is part of PlayStream - enigma2 plugin to play video streams from various sources
# Copyright (c) 2016 ivars777 (ivars777@gmail.com)
# Distributed under the GNU GPL v3. For full terms see http://www.gnu.org/licenses/gpl-3.0.en.html
# Used fragments of code from enigma2-plugin-tv3play by Taapat (https://github.com/Taapat/enigma2-plugin-tv3play)
#

import os,time,sys,os,os.path
import datetime

from enigma import ePicLoad, eServiceReference, eTimer
from Components.ActionMap import ActionMap
from Components.AVSwitch import AVSwitch
from Components.Label import Label
from Components.MenuList import MenuList
from Components.Pixmap import Pixmap
from Components.Sources.List import List
from Components.Sources.StaticText import StaticText
from Components.Button import Button
from Plugins.Plugin import PluginDescriptor
from Screens.InfoBar import MoviePlayer
from Screens.MessageBox import MessageBox
#from Screens.InputBox import InputBox
from Screens.VirtualKeyBoard import VirtualKeyBoard
from Components.Input import Input
from Screens.Screen import Screen
from Tools.BoundFunction import boundFunction
from Tools.Directories import resolveFilename, SCOPE_PLUGINS
from Tools.LoadPixmap import LoadPixmap
from ContentSources import ContentSources
from twisted.web.client import downloadPage,defer,reactor
#import enigma2_api

TMPDIR = "/tmp/playstream/"
e2 = None

#####################################################################################################################
class TVPlayer(MoviePlayer):
    def __init__(self, session, service):
        MoviePlayer.__init__(self, session, service)
        self.skinName = "MoviePlayer"

    def leavePlayer(self):
        self.close()
        #self.session.openWithCallback(self.leavePlayerConfirmed, MessageBox, _("Stop playing?"))

    def leavePlayerConfirmed(self, answer):
        if answer:
            self.close()

    def doEofInternal(self, playing):
        self.close()

    #def getPluginList(self):
        #from Components.PluginComponent import plugins
        #list = []
        #for p in plugins.getPlugins(where = PluginDescriptor.WHERE_EXTENSIONSMENU):
            #if p.name != _("TV Play"):
                #list.append(((boundFunction(self.getPluginName, p.name),
                    #boundFunction(self.runPlugin, p), lambda: True), None))
        #return list

    def showMovies(self):
        pass
    
#####################################################################################################################
class MainScreen(Screen):
    skin = """
<screen position="center,center" size="1015,570" title="Play Stream">
    <eLabel position="5,0" size="1000,2" backgroundColor="#aaaaaa" />
    <widget name="title" position="10,2" size="1000,38" font="Regular;30" />
    <widget source="list" render="Listbox" position="10,55" size="580,470" \
            scrollbarMode="showOnDemand" >
            <convert type="TemplatedMultiContent" >
            {
                    "template": [MultiContentEntryText(pos=(10, 1), size=(560, 30), \
                                    font=0, flags=RT_HALIGN_LEFT, text=0)],
                    "fonts": [gFont("Regular", 20)],
                    "itemHeight": 30
            }
            </convert>
    </widget>
    <widget name="pic" position="646,55" size="327,250" alphatest="on" />
    <widget name="cur" position="610,310" size="400,250" halign="center" font="Regular;20" />

    <ePixmap name="red"    position="100,530"   zPosition="2" size="140,40" pixmap="skin_default/buttons/red.png" transparent="1" alphatest="on" />
    <widget name="key_red" position="100,530" size="140,40" valign="center" halign="center" zPosition="4"  foregroundColor="white" font="Regular;20" transparent="1" shadowColor="background" shadowOffset="-2,-2" />
    <ePixmap name="green"  position="240,530" zPosition="2" size="140,40" pixmap="skin_default/buttons/green.png" transparent="1" alphatest="on" />
    <widget name="key_green" position="240,530" size="140,40" valign="center" halign="center" zPosition="4"  foregroundColor="white" font="Regular;20" transparent="1" shadowColor="background" shadowOffset="-2,-2" />
    <ePixmap name="yellow" position="380,530" zPosition="2" size="140,40" pixmap="skin_default/buttons/yellow.png" transparent="1" alphatest="on" />
    <widget name="key_yellow" position="380,530" size="140,40" valign="center" halign="center" zPosition="4"  foregroundColor="white" font="Regular;20" transparent="1" shadowColor="background" shadowOffset="-2,-2" />
    <ePixmap name="menu"   position="560,535" zPosition="2" size="140,40" valign="center" halign="center" pixmap="skin_default/buttons/menu.png" transparent="1" alphatest="on" />
    <widget name="key_menu" position="560,530" size="140,40" valign="center" halign="center" zPosition="4"  backgroundColor="blue" foregroundColor="white" font="Regular;20" transparent="1" shadowColor="background" shadowOffset="-2,-2" />
    
</screen>"""

    def __init__(self, session):
        Screen.__init__(self, session)
        #self.setTitle2("Home")
        self.session = session
        self.e2 = None
        self["key_red"] = Button(_("Back"))
        self["key_green"] = Button(_("Select"))
        self["key_yellow"] = Button(_("Options"))
        self["key_menu"] = Button("Menu" )
        
        self["actions"] = ActionMap(["OkCancelActions", "ColorActions","MenuActions", "NumberActions"],
                                    {
                                        "cancel": self.Cancel,
                                        "ok": self.Ok,
                                        "green": self.Ok,
                                        "red": self.Back,
                                        "yellow":self.options_screen,
                                        "menu": self.item_menu,
                                    })
        self["list"] = List([])
        self["list"].onSelectionChanged.append(self.SelectionChanged)
        self["pic"] = Pixmap()
        self["cur"] = Label()
        self["title"] = Label()
        self.downloading = 0
        if not os.path.exists(TMPDIR):
            os.mkdir(TMPDIR)
        self.onLayoutFinish.append(self.LayoutFinish)

    def LayoutFinish(self):
        self.cur_directory = os.path.dirname(os.path.realpath(__file__))
        self.defimage = LoadPixmap(os.path.join(self.cur_directory,"PlayStream.png"))
        #self.defimage = None
        #sc = AVSwitch().getFramebufferScale()
        #self.defimage0 = ePicLoad()
        #self.defimage0.PictureData.get().append(boundFunction(self.FinishDecodeDef, os.path.join(self.cur_directory,"PlayStream.png")))
        #self.defimage0.setPara((self["pic"].instance.size().width(),self["pic"].instance.size().height(),sc[0], sc[1], False, 0, "#00000000"))
        #self.defimage0.startDecode("default")
        
        self.images = {}
        self.images_url = {}
        self.picloads = {}
        self.history = []
        
        self.sources = ContentSources(os.path.join(self.cur_directory,"sources"))
        self.cur_menu = ("Home","config::home","","Sākums") #
        self.content = self.sources.get_content(self.cur_menu[1])
        print self.content
        self["list"].setList(self.content)
        self["cur"].setText(self.content[0][3])
        self.setTitle2(self.cur_menu[0])
        self.ShowPic(self.content[0][2])


    def SelectionChanged(self):
        current = self["list"].getCurrent()
        print "[PlayStream] SelectionChanged: current=",current
        if not current: return
        self["cur"].setText(current[3]) if current[3] else self["cur"].setText("")
        if current[2]:
            self.ShowPic(current[2])
        else:
            self.ShowDefPic()

    def setTitle2(self,title):
        #print self.keys()
        self["title"].setText(title)
        
    def ShowDefPic(self):
        if self.defimage:
            self["pic"].instance.setPixmap(self.defimage)

    def ShowPic(self,image_url):
        if image_url == "default":
            self.ShowDefPic()
            return
        elif self.images.has_key(image_url):
            if self.images[image_url] in (-1,-2):
                return
            elif self.images[image_url] == -3:
                self.ShowDefPic()
                return
            else:
                self["pic"].instance.setPixmap(self.images[image_url])
        else:
            if image_url.startswith("http"):
                fname = image_url.replace(":","-").replace("/","_")
                image_path = os.path.join(TMPDIR, fname)
                self.download_image(image_path, image_url)
            else: # local file
                image_path = os.path.join(self.cur_directory,image_url)
                self.start_decode(image_path,image_url)
            
    def start_decode(self,image_path,image_url):
        self.images[image_url] = -2
        self.images_url[image_path] = image_url
        sc = AVSwitch().getFramebufferScale()
        if  not self.picloads.has_key(image_path):
            self.picloads[image_path] = ePicLoad()
            self.picloads[image_path].PictureData.get().append(boundFunction(self.FinishDecode, image_path))
            self.picloads[image_path].setPara((self["pic"].instance.size().width(),
                                          self["pic"].instance.size().height(),
                                          sc[0], sc[1], False, 0, "#00000000"))
            print image_path,image_url
            self.picloads[image_path].startDecode(image_path)

    def FinishDecode(self, image_path,picInfo = None):
        image_url = self.images_url[image_path]
        del self.images_url[image_path] #III
        self.images[image_url] = self.picloads[image_path].getData()
        self["pic"].instance.setPixmap(self.images[image_url])
        del self.picloads[image_path]
        if len(self.images)>30:
            del self.images[self.images.keys()[0]]
            # self.images.pop()
            
    #def FinishDecodeDef(self, image_path,picInfo = None):
    #    self.defimage = self.defimage0.getData()
    #    del self.defimage0
    #    self["pic"].instance.setPixmap(self.defimage)
        
    def download_image(self,image_path,image_url):
        print "Image download started",self.downloading,image_path,image_url
        self.downloading += 1
        self.images[image_url] = -1
        downloadPage(image_url, image_path).addCallback(boundFunction(self.downloadFinished, image_path,image_url)).addErrback(boundFunction(self.downloadFailed, image_path,image_url))

    def downloadFinished(self, image_path, image_url, result):
        self.downloading -= 1
        print "[TV Play] Image downloaded finished ",self.downloading,image_path, image_url,result
        self.start_decode(image_path,image_url)       

    def downloadFailed(self, image_path, image_url,result):
        self.downloading -= 1
        print "[TV Play] Image downloaded failed ",self.downloading,image_path, image_url,result
        self.images[image_url] = -3


    def Ok(self):
        current = self["list"].getCurrent()
        index = self["list"].getIndex()
        
        if current[1].startswith("http") or current[1].startswith("rstp://") or self.sources.is_video(current[1]):
            if self.sources.is_video(current[1]):
                try:
                    current = self.sources.get_content(current[1])
                    if not (current[1].startswith("http") or current[1].startswith("rstp://")):
                        self.session. open(MessageBox, "Can not play stream- %s"%current[0] , MessageBox.TYPE_INFO)
                        return
                except Exception,e:
                    self.session. open(MessageBox, "Can not play stream - %s"%str(e) , MessageBox.TYPE_INFO)
                    return
            self.playVideo(current)
            
        elif current[1] == "back":
            cur_menu_old = self.cur_menu
            self.cur_menu = self.history.pop()
            new_content = self.sources.get_content(self.cur_menu[1])
            try:
                index = zip(*new_content)[1].index(cur_menu_old[1])
            except:
                index = 0
            self.setTitle2(self.cur_menu[0])
            self.show_content(new_content,index)
            
        else:
            print "selected=",current        
            self.history.append(self.cur_menu)
            self.cur_menu = current
            try:
                new_content = self.sources.get_content(self.cur_menu[1])
            except Exception,e:
                self.cur_menu = self.history.pop()                
                self.session. open(MessageBox, "Error - %s"%str(e) , MessageBox.TYPE_INFO)
                return
            self.setTitle2(self.cur_menu[0])
            self.show_content(new_content)
 
    def Back(self):
        self["list"].setIndex(0)
        self.Ok()

    def show_content(self,content,index=0):
        self["list"].setList(content)
        self["list"].setIndex(index)
        self.SelectionChanged()

    def Cancel(self):
        #if os.path.exists(TMPDIR):
            #for name in os.listdir(TMPDIR):
                #os.remove(os.path.join(TMPDIR, name))
            #os.rmdir(TMPDIR)
        self.close()
            
    def playVideo(self, current):
        hlspath = current[1]
        print "[TV Play] Play:", current
        ref = eServiceReference(4097, 0, hlspath)
        ref.setName(current[0])
        self.session.open(TVPlayer, ref)
    
    def item_menu(self):
        print "\n[PlayStream] options\n"
        current = self["list"].getCurrent()
        index = self["list"].getIndex()        
        #self.session. open(MessageBox, "Item options - %s"%current[0] , MessageBox.TYPE_INFO)
        args=[current,self.cur_menu]
        self.session.open(ItemMenuScreen,current,index,self)
    
    def options_screen(self):
        source = self.cur_menu[1].split("::")[0]
        options = self.sources.options_read(source)
        print source
        if not options:
            self.session. open(MessageBox, "No options available for source %s (%s)"%(self.cur_menu[0],source) , MessageBox.TYPE_INFO)
        else:
            self.session.open(OptionsScreen,self)        
        
        
#####################################################################################################################
class ItemMenuScreen(Screen):
    skin = """
<screen position="center,center" size="420,420" title="Item menu" >
    <widget name="list" position="10,0" size="400,400" scrollbarMode="showOnDemand" />
    <widget name="info" position="10,400" size="400,20" halign="left" font="Regular;18" />
</screen>"""

    def __init__(self, session, *args):
        Screen.__init__(self, session)
        self.session = session
        self.cur_item = args[0]
        self.cur_index = args[1]
        self.main = args[2]
        #print args[0],args[1]

        self.config = self.main.sources.plugins["config"]
        #print self.config.name
        #print "\n[PlayStream] ItemMenuScreen %s | %s\n"%(args[0][1],args[1])
        self['actions'] = ActionMap(
            ['OkCancelActions', "ColorActions"],
            {'ok': self.run,
             'cancel': self.close,
             "red":self.close}, 
            -1)
        #self["list"] = List([])
        lst = [
            ("Aditional information","info","Display additional information about item"),
            ("Add to bouquet","bouquet","Add current item to Enigma2 bouquet"),
            ("Add to favorites","favorites","Add current item to PlayStrem favorites"),
            ("Download video","download","Download video in background")
        ]
        if "config::" in self.main.cur_menu[1]:
            lst.extend([
            ("Rename item","rename","Rename list item"),
            ("Move item","move","Move list item"),
            ("Delete item","delete","Delete list item"),
            ("Add submenu","add_list","Add submenu before selected item"), 
            ])
        self['list'] = MenuList(lst)
        self["list"].onSelectionChanged.append(self.SelectionChanged)        
        self["info"] = Label()
        #self["list"].setList(lst)
        self.setTitle(self.cur_item[0])
        self["info"].setText(lst[0][2])
        #self.onLayoutFinish.append(self.LayoutFinish)
        
    def LayoutFinish(self):
        pass

    def SelectionChanged(self):
        current = self["list"].getCurrent()
        print "[PlayStream item menu] SelectionChanged: current=",current
        if not current: return
        self["info"].setText(current[2])


    def run(self):
        current = self['list'].getCurrent()
        if current[1] == "info":
            self.session.open(MessageBox, "Not yet implemented!", MessageBox.TYPE_INFO)
            pass # TODO parada papildus info
        
        elif current[1] == "bouquet":
            #if not e2:
                #e2 = enigma2_api.DBServices()
            #print "load_buuquets - ",e2._load_bouquets()
            self.session.open(MessageBox, "Not yet implemented!", MessageBox.TYPE_INFO)            
            
        elif current[1] == "favorites":
            lists = self.config.get_lists()
            lists2 = [(l,l) for l in lists]            
            self.session.openWithCallback(self.cb_favorites,ListBox, "Select menu",lists2) 
                        
        elif current[1] == "download":
            self.session.open(MessageBox, "Not yet implemented!", MessageBox.TYPE_INFO)            
            pass #TODO

        elif current[1] == "delete":
            lst = self.main.cur_menu[1].replace("config::","")
            #print lst
            self.config.del_item(lst,self.cur_index)
            self.config.write_streams()
            txt = "'%s' deleted from favourite stream list '%s'"%(self.cur_item[0],lst)
            self.session.open(MessageBox, txt, MessageBox.TYPE_INFO)  
            
        elif current[1] == "rename":
            #name2 = "Renamed"
            self.session.openWithCallback(self.cb_rename,VirtualKeyBoard, title="Enter new item name", text=self.cur_item[0]) 
            
        elif current[1] == "add_list":
            self.session.open(MessageBox, "Not yet implemented!", MessageBox.TYPE_INFO)            
            pass #TODO    
        return
    
    def cb_rename(self,value):
        if not value:
            return
        lst = self.main.cur_menu[1].replace("config::","")
        pos = self.cur_index        
        print value
        item2 = list(self.cur_item)
        item2[0]=value
        item2 = tuple(item2)
        self.config.replace_item(lst,item2,pos)
        self.config.write_streams()
        txt = "'%s' renamed to '%s'"%(self.cur_item[0],value)
        self.session.open(MessageBox, txt, MessageBox.TYPE_INFO)     
        
    def cb_favorites(self,value):
        if not value:
            return 
        self.config.add_item(value,self.cur_item)
        self.config.write_streams()
        txt = "'%s' added to favourite stream list '%s'"%(self.cur_item[0],value)
        self.session.open(MessageBox, txt, MessageBox.TYPE_INFO)
        #self.session.openWithCallback(self.callMyMsg, MessageBox, _("Do you want to exit the plugin?"), MessageBox.TYPE_INFO)
        
class ListBox(Screen):
    skin = """
<screen position="center,center" size="420,420" title="ListBox" >
    <widget name="list" position="10,0" size="400,400" scrollbarMode="showOnDemand" />
</screen>"""

    def __init__(self, session, title="ListBox",lst=[]):
        Screen.__init__(self, session)
        self.session = session
        self.lst = lst
        self['actions'] = ActionMap(
            ['OkCancelActions', "ColorActions"],
            {'ok': self.go,
             'cancel': self.close,
             "red":self.close,
             "green":self.go}, 
            -1)
        self['list'] = MenuList(lst)
        self.setTitle(title)
        #self.onLayoutFinish.append(self.LayoutFinish)
        
    #def LayoutFinish(self):
        #pass
        
    def go(self):
        self.current = self['list'].getCurrent()
        self.close(self.current[0])

    def cancel(self):
        self.close(None)
    
##########################################################################
from Components.config import config, ConfigSubsection, ConfigYesNo,\
     getConfigListEntry, ConfigSelection, ConfigNumber, ConfigDirectory,ConfigText, ConfigSubDict
from Components.ConfigList import ConfigListScreen
#from Screens.LocationBox import LocationBox

config.plugins.playstream = ConfigSubDict()

class OptionsScreen(ConfigListScreen,Screen):
    skin = """
<screen position="center,center" size="560,400"  >
	<ePixmap name="red"    position="0,0"   zPosition="2" size="140,40" pixmap="skin_default/buttons/red.png" transparent="1" alphatest="on" />
	<widget name="key_red" position="0,0" size="140,40" valign="center" halign="center" zPosition="4"  foregroundColor="white" font="Regular;20" transparent="1" shadowColor="background" shadowOffset="-2,-2" />
	<ePixmap name="green"  position="140,0" zPosition="2" size="140,40" pixmap="skin_default/buttons/green.png" transparent="1" alphatest="on" />
	<widget name="key_green" position="140,0" size="140,40" valign="center" halign="center" zPosition="4"  foregroundColor="white" font="Regular;20" transparent="1" shadowColor="background" shadowOffset="-2,-2" />
	<ePixmap name="yellow" position="280,0" zPosition="2" size="140,40" pixmap="skin_default/buttons/yellow.png" transparent="1" alphatest="on" />
	<widget name="key_yellow" position="280,0" size="140,40" valign="center" halign="center" zPosition="4"  foregroundColor="white" font="Regular;20" transparent="1" shadowColor="background" shadowOffset="-2,-2" />
	<ePixmap name="blue"   position="420,0" zPosition="2" size="140,40" pixmap="skin_default/buttons/blue.png" transparent="1" alphatest="on" />
	<widget name="key_blue" position="420,0" size="140,40" valign="center" halign="center" zPosition="4"  foregroundColor="white" font="Regular;20" transparent="1" shadowColor="background" shadowOffset="-2,-2" />

	<widget name="config" position="10,40" size="540,340" scrollbarMode="showOnDemand" />
</screen>"""

    def __init__(self, session,*args):
        self.session = session
        Screen.__init__(self, session)
        self.main = args[0]
        self.setTitle(self.main.cur_menu[0]+" Options")
        self.source = self.main.cur_menu[1].split("::")[0]
        self.cfg = config.plugins.playstream
        self.list = []       
        self.options = self.main.sources.options_read(self.source)
        if not self.options:
            #self.session. open(MessageBox, "No options available for source %s (%s)"%(self.main.cur_menu[0],self.source) , MessageBox.TYPE_INFO)
            self.close(False,self.session)
        for k in self.options:
            self.cfg[k]=ConfigText(default=self.options[k],fixed_size=False)
            self.list.append(getConfigListEntry(k, self.cfg[k]))
        ConfigListScreen.__init__(self, self.list, session = self.session)
        
        self["key_red"] = Button(_("Cancel"))
        self["key_green"] = Button("Save")
        self["key_yellow"] = Button("")
        self["key_blue"] = Button("")
        self["setupActions"] = ActionMap(["SetupActions", "ColorActions"],
                                         {
                                             "red": self.cancel,
                                             "green": self.save,
                                             "save": self.save,
                                             "cancel": self.cancel,
                                             "ok": self.ok,
                                             }, -2)

    def getCurrentEntry(self):
        return self["config"].getCurrent()[0]

    def getCurrentValue(self):
        return str(self["config"].getCurrent()[1].getText())

    def ok(self):
        self.save()
        #if self["config"].getCurrent()[1] == config.plugins.getpicons.folder:
            #folder  = config.plugins.getpicons.folder.value
            #self.session.openWithCallback(self.change_dir, LocationBox,"Select Folder")
        #else:
            

    #def change_dir(self, folder, select=None):
        #if folder:
            ##print "change_dir to %s"%folder
            #config.plugins.getpicons.folder.value = folder

    def save(self):
        print "saving"
        #self.saveAll()
        for k in self.options.keys():
            self.options[k]=self.cfg[k].value
            print "%s=%s"%(k,self.cfg[k].value)
        self.main.sources.options_write(self.source,self.options)
        self.close(True,self.session)

    def cancel(self):
        print "cancel"
        self.close(False,self.session)
