#!/usr/bin/env python
# coding=utf8
#
# This file is part of PlayStream - enigma2 plugin to play video streams from various sources
# Copyright (c) 2016 ivars777 (ivars777@gmail.com)
# Distributed under the GNU GPL v3. For full terms see http://www.gnu.org/licenses/gpl-3.0.en.html
#

import urllib2, urllib
import datetime, re, sys,os
from collections import OrderedDict
import ConfigParser

headers2dict = lambda  h: dict([l.strip().split(": ") for l in h.strip().splitlines()])
headers0 = headers2dict("""
User-Agent: Mozilla/5.0 (Linux; U; Android 4.4.4; Nexus 5 Build/KTU84P) AppleWebkit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30
""")
    
class SourceBase(object):
    """Stream source base class"""
    
    def __init__(self,country="lv"):
        self.name = "name"
        self.title = "Title"
        self.img = ""
        self.desc = ""
        self.options = OrderedDict()
        self.config_file = ""
        
    def options_read(self):
        return None
    
    def options_write(self,options):
        return
                
    def login(self,user="",password=""):
        return False
    
    def options_read(self):
        if not ("options" in dir(self) and self.options): # process options only if self.options defined, self.config_file should be defined too
            return None
        config = ConfigParser.ConfigParser()        
        if os.path.exists(self.config_file):
            config.read(self.config_file)
            self.options = OrderedDict(config.items(self.name))             
        else:
            self.options_write(self.options)
        return self.options
    
    def options_write(self,options):        
        config = ConfigParser.ConfigParser() 
        config.add_section(self.name)
        for k in options.keys():
            config.set(self.name, k,options[k])
        with open(self.config_file,"w") as f:
            config.write(f)
        self.options = OrderedDict(config.items(self.name))        

    def logout(self):
        return False
            
    def get_content(self, data):
        return None
    
    def is_video(self,data):
        return False
        
    def call(self, data,headers=headers0,lang=""):
        return None

    def _http_request(self, url,params = None, headers=None):
        if not headers: headers = headers0
        try:
            r = urllib2.Request(url, data=params, headers=headers)
            u = urllib2.urlopen(r)
            content = u.read()
            u.close()
            return content
        except Exception as ex:
            return None

if __name__ == "__main__":
    pass